/*
Example MQTT-switch-relay-node with 4 buttons and 4 leds

    connects to an MQTT server

    publishes “hello world" to the topic

 

    sends a specific “on"/“off" to the topic a specific button is pressed

    multiple arduino’s with same generic sketch can run parallel to each other

    multiple arduino’s need each to have a unique ip-addres, unique mac address and unique MQTT client-ID

    tested on arduino-uno with W5100 ethernet shield

    Ethernet Shield W5100 uses pins 4,10,11,12,13

    availbale digital pins: 1,2,3,5,6,7,8,9
    */

//------------------------------------------------------------------------------

#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>


// Set led variables to Arduino digital pins
const byte relay1 = PD2; //pin 2
const byte relay2 = PD3; //pin 3
const byte relay3 = PD5; //pin 5 -  4 used by ethernet shield
const byte relay4 = PD6; //pin 6
const byte relay5 = PD7; //pin 7
const byte relay6 = PB0; //pin 8
const byte relay7 = PB1; //pin 9
const byte relay8 = PC0; //pin A0



//---------------------------------------------------------------------------

// Arduino MAC address is on a sticker on your Ethernet shield
// must be unique for every node in same network
// To make a new unique address change last letter

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xEE };

// Unique static IP address of this Arduino - change to adapt to your network
IPAddress ip(192,168,0,238);

// IP Address of your MQTT broker - change to adapt to your network
byte server[] = { 192, 168, 0, 20 };



void callback(char*, byte*, unsigned int);
void reconnect();


// Initiate instances -----------------------------------

EthernetClient ethClient;
PubSubClient client(server, 1883, callback, ethClient);



//-------------------------------------------------------

void setup()

{

    DDRB = DDRB | B00000011;
    DDRC = DDRC | B00000001;
    DDRD = DDRD | B11101100;



// Setup serial connection
Serial.begin(9600);

// Setup ethernet connection to MQTT broker
Ethernet.begin(mac,ip);
    if (client.connect("arduino-ip-238")) { // change as desired - clientname must be unique for MQTT broker
       // client.publish("status/relay","hello world - here arduino ip 239");
        Serial.println("connected");
        client.subscribe("control/relay");
    }

    while(true){
       if (!client.connected()) {
          reconnect();
       }
      client.loop();
    }

}



//----------------------------------------------

void loop()
{}


void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");

    // Attempt to connect
   if (client.connect("arduino-ip-238")) {
      Serial.println("connected"); 
      // Subscribe or resubscribe to a topic
      // You can subscribe to more topics (to control more outputs)
      client.subscribe("control/relay"); 

    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds time after  retrying
      delay(5000);
      Serial.println(Ethernet.localIP());
    }
  }
}


void callback(char* topic, byte* payload, unsigned int length) {
    // handle message arrived
    String content="";
    char character;

    for (unsigned int num=0;num<length;num++) {
        character = payload[num];
        content.concat(character);
    }

    Serial.print(topic);
    Serial.print(" payload: ");
    Serial.println(content);

    if (content == "1on") {
        PORTD = PORTD | (1 << relay1);
        //digitalWrite(relay1,HIGH);
        client.publish("state/relay","1on");

    }

    if (content == "1off") {
        PORTD = PORTD & ~(1 << relay1);
        //digitalWrite(relay1, LOW);
        client.publish("state/relay","1off");

    }

    if (content == "2on") {
        PORTD = PORTD | (1 << relay2);
        //digitalWrite(relay2, HIGH);
        client.publish("state/relay","2on");

    }

    if (content == "2off") {
        PORTD = PORTD & ~(1 << relay2);
        //digitalWrite(relay2, LOW);
        client.publish("state/relay","2off");
    }

    if (content == "3on") {
        PORTD = PORTD | (1 << relay3);
        //digitalWrite(relay3, HIGH);
        client.publish("state/relay","3on");
    }

    if (content == "3off") {
        PORTD = PORTD & ~(1 << relay3);
        //digitalWrite(relay3, LOW);
         client.publish("state/relay","3off");

    }

    if (content == "4on") {
        PORTD = PORTD | (1 << relay4);
        //digitalWrite(relay4, HIGH);
        client.publish("state/relay","4on");

    }

    if (content == "4off") {
        PORTD = PORTD & ~(1 << relay4);
        //digitalWrite(relay4, LOW);
        client.publish("state/relay","4off");

    }

    if (content == "5on") {
        PORTD = PORTD | (1 << relay5);
        //digitalWrite(relay5, HIGH);
        client.publish("state/relay","5on");

    }

    if (content == "5off") {
        PORTD = PORTD & ~(1 << relay5);
        //digitalWrite(relay5, LOW);
        client.publish("state/relay","5off");

    }

    if (content == "6on") {
        PORTB = PORTB | (1 << relay6);
        //digitalWrite(relay6, HIGH);
        client.publish("state/relay","6on");

    }

    if (content == "6off") {
        PORTB = PORTB & ~(1 << relay6);
        //digitalWrite(relay6, LOW);
        client.publish("state/relay","6off");

    }

    if (content == "7on") {
        PORTB = PORTB | (1 << relay7);
        //digitalWrite(relay7, HIGH);
        client.publish("state/relay","7on");

    }

    if (content == "7off") {
        PORTB = PORTB & ~(1 << relay7);
        //digitalWrite(relay7, LOW);
        client.publish("state/relay","7off");

    }

    if (content == "8on") {
        PORTC = PORTC | (1 << relay8);
        //digitalWrite(relay8, HIGH);
        client.publish("state/relay","8on");

    }

    if (content == "8off") {
        PORTC = PORTC & ~(1 << relay8);
        //digitalWrite(relay8, LOW);
        client.publish("state/relay","8off");

    }

    
}

// End of sketch ---------------------------------
